##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Form Framework Interfaces.

$Id: $
"""
__docformat__ = "reStructuredText"
import zope.interface
import zope.schema

from z3c.form.interfaces import IButton, IButtonHandler, IManager, IWidget
from z3c.form.interfaces import ISelectionManager, IForm


# -----[ Event Subscription ]------------------------------------------------

class IJSEvent(zope.interface.Interface):
    """An interface for javascript event objects."""

    name = zope.schema.TextLine(
        title=u"Event Name",
        description=u"The name of an event (i.e. click/dblclick/changed).",
        required=True)


class ISelector(zope.interface.Interface):
    """An object describing the selection of DOM Elements."""

class IIdSelector(ISelector):
    """Select a DOM element by Id."""

    id = zope.schema.TextLine(
        title=u"Id",
        description=u"Id of the DOM element to be selected.",
        required=True)

class ICSSSelector(ISelector):
    """Select a DOM element by a CSS selector expression."""

    expr = zope.schema.TextLine(
        title=u"Expression",
        description=u"CSS selector pointing to a DOM element.",
        required=True)


class IJSSubscription(zope.interface.Interface):
    """A Subscription within Javascript."""

    event = zope.schema.Object(
        title=u"Event",
        description=u"The event.",
        schema = IJSEvent,
        required=True)

    selector = zope.schema.Object(
        title=u"Selector",
        description=u"The DOM element selector.",
        schema = ISelector,
        required=True)

    handler = zope.schema.Field(
        title=u"Handler",
        description=(u"A callable nneding three argument: event, selector, "
                     u"and request."),
        required=True)


class IJSSubscriptions(zope.interface.Interface):
    """A manager of Javascript event subscriptions."""

    def subscribe(event, selector, handler):
        """Subscribe an event for a DOM element executing the handler's
        result."""

    def __iter__():
        """Return an iterator of all subscriptions."""

    def __getitem__(name):
        """Return all the subscriptions for the handler with the given name."""


class IRenderer(zope.interface.Interface):
    """Render a component in the intended output format."""

    def update():
        """Update renderer."""

    def render():
        """Render content."""

# -----[ Javascript Functions ]----------------------------------------------

class IJSFunction(zope.interface.Interface):
    """A Javascript Function."""

    name = zope.schema.BytesLine(
        title=u"Name",
        description=u"The name of the function.",
        required=True)

    arguments = zope.schema.List(
        title=u"Arguments",
        description=u"A list of arguments of the function.",
        required=True)

    def render():
        """Render the content of the JS function."""

    def call():
        """Render a JavaScript call to the function."""


class IJSFunctions(zope.interface.Interface):
    """A manager of Javascript functions."""

    def add(function, namespace=''):
        """Add a new function to the given namespace."""

    def render(self):
        """Render all functions."""

class IHaveJSFunctions(zope.interface.Interface):
    """An component that has a JS functions manager .

    This component is most often a view component. When rendering a page this
    interface is used to check whether any functions must be rendered.
    """

    jsFunctions = zope.schema.Object(
        title=u"Javascript Functions",
        description=u"Attribute holding the JS Functions Manager.",
        schema = IJSFunctions,
        required=True)

# -----[ Widgets ]------------------------------------------------------------

class IWidgetSelector(ISelector):
    """Select a DOM element using the action."""

    action = zope.schema.Field(
        title=u"Action",
        description=u"The action being selected.",
        required=True)

# -----[ Views and Forms ]----------------------------------------------------

class IHaveJSSubscriptions(zope.interface.Interface):
    """An component that has a subscription manager .

    This component is most often a view component. When rendering a page this
    interface is used to check whether any subscriptions must be rendered.
    """

    jsSubscriptions = zope.schema.Object(
        title=u"Javascript Subscriptions",
        description=u"Attribute holding the JS Subscription Manager.",
        schema = IJSSubscriptions,
        required=True)


# -----[ Buttons and Handlers ]----------------------------------------------


class IJSButton(IButton):
    """A button that just connects to javascript handlers."""


class IJSEventHandler(zope.interface.Interface):
    """An action handler of Javascript events for fields and buttons."""

    __name__ = zope.schema.ASCIILine(
        title=u"Name",
        description=u"The name of the handler. Like a function's name",
        required=True)

    def __call__(event, selector, request):
        """Call the handler.

        The result should be a string containing the script to be executed
        when the event occurs on the specified DOM element.

        The event *must* provide the ``IJSEvent`` interface. The selector
        *must* be an ``IWidgetSelector`` component.
        """


class IJSEventHandlers(zope.interface.Interface):
    """Javascript event handlers for fields and buttons."""

    def addHandler(field, event, handler):
        """Add a new handler for the fiven field and event specification.

        The ``field`` and ``event`` arguments can either be instances, classes
        or specifications/interfaces. The handler *must* provide the
        ``IJSEventHandler`` interface.
        """

    def getHandlers(field):
        """Get a list of (event, handler) pairs for the given field.

        The event is a component providing the ``IJSEvent`` interface, and the
        handler is the same component that has been added before.
        """

    def copy():
        """Copy this object and return the copy."""

    def __add__(other):
        """Add another handlers object.

        During the process a copy of the current handlers object should be
        created and the other one is added to the copy. The return value is
        the copy.
        """


# -----[ Validator ]--------------------------------------------------------


class IValidationScript(zope.interface.Interface):
    """Component that renders the script doing the validation."""

    def render():
        """Render the js expression."""

class IMessageValidationScript(IValidationScript):
    """Causes a message to be returned at validation."""


class IAJAXValidator(zope.interface.Interface):
    """A validator that sends back validation data sent from an ajax request."""

    ValidationScript = zope.schema.Object(
        title=u"Validation Script",
        schema=IValidationScript)

    def validate():
        """Return validation data."""


# -----[ Widget Mode Switcher ]-----------------------------------------------

class IWidgetModeSwitcher(zope.interface.Interface):
    """A component that enables forms to switch between display and input
    widgets."""

    def getDisplayWidget():
        """Return the rendered display widget.

        The method expects to find a field called 'widget-name' in the request
        containing the short name to the field/widget.
        """

    def getInputWidget():
        """Return the rendered input widget.

        The method expects to find a field called 'widget-name' in the request
        containing the short name to the field/widget.
        """

    def saveWidgetValue():
        """Save the new value of the widget and return any possible errors.

        The method expects to find a field called 'widget-name' in the request
        containing the short name to the field/widget. The request must also
        contain all fields required for the widget to successfully extract the
        value.
        """

class IWidgetSwitcher(zope.interface.Interface):
    """Switches the widhet to another mode."""

    form = zope.schema.Field(
        title=u"Form",
        description=u"The form.",
        required=True)

    widget = zope.schema.Field(
        title=u"Widget",
        description=u"The widget that is being switched.",
        required=True)

    mode = zope.schema.TextLine(
        title=u"Mode",
        description=u"The mode to which to switch to.",
        required=True)

    def render():
        """Render the switcher into JavaScript."""

class ILabelWidgetSwitcher(zope.interface.Interface):
    """Switches the widget to an input widget when clicking on the label."""

    form = zope.schema.Field(
        title=u"Form",
        description=u"The form.",
        required=True)

    mode = zope.schema.TextLine(
        title=u"Mode",
        description=u"The mode to which to switch to.",
        required=True)

    def render():
        """Render the switcher into JavaScript."""


class IWidgetSaver(zope.interface.Interface):
    """Saves a widget's value to the server."""

    form = zope.schema.Field(
        title=u"Form",
        description=u"The form.",
        required=True)

    widget = zope.schema.Field(
        title=u"Widget",
        description=u"The widget for which the value is saved.",
        required=True)

    def render():
        """Render the saver into JavaScript."""


# -----[ AJAX ]--------------------------------------------------------


class IAJAXRequestHandler(zope.interface.Interface):
    """An object that has methods for handling ajax requests."""

    ajaxRequestHandlers = zope.schema.Object(
        title=u"AJAX Request Handlers Manager",
        schema=ISelectionManager)


class IAJAXHandler(zope.interface.Interface):

    def __call__(context, request):
        """Return a callable that calls the handler function.

        Return a callable which has access to context and request
        without context and request being passed as arguments.
        """


# -----[ Form Traverser ]-------------------------------------------------

class IFormTraverser(zope.interface.Interface):
    """Marker interface for forms that can be traversed by the @@ajax
    view."""



# -----[ Client Side Event System ]---------------------------------------

class IClientEventHandlers(zope.interface.Interface):
    """A collection of client side event handlers for server side events."""

    def addHandler(required, handler):
        """Add a new handler for a the given required interfaces."""

    def getHandler(event):
        """Get the handler for the given server side event."""

    def copy():
        """Copy this object and return the copy."""

    def __add__(other):
        """Add another handlers object.

        During the process a copy of the current handlers object should be
        created and the other one is added to the copy. The return value is
        the copy.
        """

class IClientEventHandler(zope.interface.Interface):
    """A handler managed by the client event handlers."""

    def __call__(self, form, event):
        """Execute the handler."""



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Form Framework Interfaces.

$Id: test_doc.py 79169 2007-08-23 19:08:29Z srichter $
"""
__docformat__ = "reStructuredText"
import unittest
import zope.testing.doctest

from z3c.form import testing

def tearDown(test):
    testing.tearDown(test)
    from zope.app.pagetemplate import engine
    engine.TrustedEngine = engine._TrustedEngine()

def test_suite():
    return unittest.TestSuite((
        zope.testing.doctest.DocFileSuite(
            '../jsevent.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../ajax.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../jsfunction.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../jsaction.txt',
            setUp=testing.setUp, tearDown=tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../jsclientevent.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../jqueryrenderer.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../jsvalidator.txt',
            setUp=testing.setUp, tearDown=testing.tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        zope.testing.doctest.DocFileSuite(
            '../jsswitch.txt',
            setUp=testing.setUp, tearDown=tearDown,
            optionflags=zope.testing.doctest.NORMALIZE_WHITESPACE |
                        zope.testing.doctest.ELLIPSIS),
        ))



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Form Framework Event Framework.

$Id: $
"""
__docformat__ = "reStructuredText"
import sys
import zope.component
import zope.interface
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.viewlet import viewlet

from z3c.formjs import interfaces


class JSEvent(object):
    """Javascript Event"""
    zope.interface.implements(interfaces.IJSEvent)

    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return '<JSEvent "%s">' % self.name


CLICK = JSEvent("click")
DBLCLICK = JSEvent("dblclick")
CHANGE = JSEvent("change")
LOAD = JSEvent("load")
BLUR = JSEvent("blur")
FOCUS = JSEvent("focus")
KEYDOWN = JSEvent("keydown")
KEYUP = JSEvent("keyup")
MOUSEDOWN = JSEvent("mousedown")
MOUSEMOVE = JSEvent("mousemove")
MOUSEOUT = JSEvent("mouseout")
MOUSEOVER = JSEvent("mouseover")
MOUSEUP = JSEvent("mouseup")
RESIZE = JSEvent("resize")
SELECT = JSEvent("select")
SUBMIT = JSEvent("submit")

EVENTS = (CLICK, DBLCLICK, CHANGE, LOAD, BLUR, FOCUS, KEYDOWN, KEYUP,
          MOUSEDOWN, MOUSEMOVE, MOUSEOUT, MOUSEOVER, MOUSEUP, RESIZE, SELECT,
          SUBMIT)

class IdSelector(object):
    zope.interface.implements(interfaces.IIdSelector)

    def __init__(self, id):
        self.id = id

    def __repr__(self):
        return '<%s "%s">' %(self.__class__.__name__, self.id)


class CSSSelector(object):
    zope.interface.implements(interfaces.ICSSSelector)

    def __init__(self, expr):
        self.expr = expr

    def __repr__(self):
        return '<%s "%s">' %(self.__class__.__name__, self.expr)


class JSSubscription(object):
    zope.interface.implements(interfaces.IJSSubscription)

    def __init__(self, event, selector, handler):
        self.event = event
        self.selector = selector
        self.handler = handler

    def __repr__(self):
        return '<%s event=%r, selector=%r, handler=%r>' % (
            self.__class__.__name__, self.event, self.selector, self.handler)


class JSSubscriptions(object):
    zope.interface.implements(interfaces.IJSSubscriptions)

    def __init__(self):
        self._subscriptions = {}

    def subscribe(self, event, selector, handler):
        subscription = JSSubscription(event, selector, handler)
        name = handler.__name__
        subscriptions = self._subscriptions.get(name, [])
        subscriptions.append(subscription)
        self._subscriptions[name] = subscriptions
        return subscription

    def __iter__(self):
        for subscriptions in self._subscriptions.values():
            for subscription in subscriptions:
                yield subscription

    def __getitem__(self, name):
        return self._subscriptions[name]


def subscribe(selector, event=CLICK):
    """A decorator for defining a javascript event handler."""
    def createSubscription(func):
        frame = sys._getframe(1)
        f_locals = frame.f_locals
        subs = f_locals.setdefault('jsSubscriptions', JSSubscriptions())
        subs.subscribe(event, selector, func)
        return func
    return createSubscription


class JSSubscriptionsViewlet(viewlet.ViewletBase):
    """An viewlet for the JS viewlet manager rendering subscriptions."""
    zope.component.adapts(
        zope.interface.Interface,
        IBrowserRequest,
        interfaces.IHaveJSSubscriptions,
        zope.interface.Interface)

    # This viewlet wants to be very heavy, so that it is rendered after all
    # the JS libraries are loaded.
    weight = 1000

    def update(self):
        self.renderer = zope.component.getMultiAdapter(
            (self.__parent__.jsSubscriptions, self.request),
            interfaces.IRenderer)
        self.renderer.update()

    def render(self):
        content = self.renderer.render()
        return u'<script type="text/javascript">\n%s\n</script>' % content



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Form Framework Button Framework.

$Id: $
"""
__docformat__ = "reStructuredText"
import sys
import zope.component
import zope.interface
import zope.location
from zope.interface import adapter
from z3c.form import button, action, util
from z3c.form.browser.button import ButtonWidget
from z3c.form.interfaces import IField, IFieldWidget
from z3c.form.interfaces import IFormLayer, IFormAware
from z3c.form.interfaces import IButtonAction, IAfterWidgetUpdateEvent

from z3c.formjs import interfaces, jsevent


class WidgetSelector(jsevent.IdSelector):
    zope.interface.implements(interfaces.IWidgetSelector)

    def __init__(self, widget):
        self.widget = widget

    @property
    def id(self):
        return self.widget.id


class JSButton(button.Button):
    """A simple javascript button in a form."""
    zope.interface.implements(interfaces.IJSButton)


class JSButtonAction(action.Action, ButtonWidget, zope.location.Location):
    """A button action specifically for JS buttons."""
    zope.interface.implements(IButtonAction)
    zope.component.adapts(IFormLayer, interfaces.IJSButton)

    def __init__(self, request, field):
        action.Action.__init__(self, request, field.title)
        ButtonWidget.__init__(self, request)
        self.field = field

    @property
    def accesskey(self):
        return self.field.accessKey

    @property
    def value(self):
        return self.title

    @property
    def id(self):
        return self.name.replace('.', '-')


class JSHandlers(object):
    """Advanced Javascript event handlers for fields and buttons.

    This implementation of the Javascript event handlers interface used an
    adapter registry to manage more general versus more specific handler
    registrations. When asked for handlers, the registry will always return
    the most specific one for each event.
    """
    zope.interface.implements(interfaces.IJSEventHandlers)

    def __init__(self):
        self._registry = adapter.AdapterRegistry()
        self._handlers = ()

    def addHandler(self, field, event, handler):
        """See interfaces.IJSEventHandlers"""
        # Create a specification for the field and event
        fieldSpec = util.getSpecification(field)
        eventSpec = util.getSpecification(event)
        if isinstance(fieldSpec, util.classTypes):
            fieldSpec = zope.interface.implementedBy(fieldSpec)
        if isinstance(eventSpec, util.classTypes):
            eventSpec = zope.interface.implementedBy(eventSpec)
        # Register the handler
        self._registry.register(
            (fieldSpec, eventSpec), interfaces.IJSEventHandler, '', handler)
        self._handlers += ((field, event, handler),)

    def getHandlers(self, field):
        """See interfaces.IJSEventHandlers"""
        fieldProvided = zope.interface.providedBy(field)
        handlers = ()
        for event in jsevent.EVENTS:
            eventProvided = zope.interface.providedBy(event)
            handler = self._registry.lookup(
                (fieldProvided, eventProvided), interfaces.IJSEventHandler)
            if handler:
                handlers += ((event, handler),)
        return handlers

    def copy(self):
        """See interfaces.IJSEventHandlers"""
        handlers = JSHandlers()
        for field, event, handler in self._handlers:
            handlers.addHandler(field, event, handler)
        return handlers

    def __add__(self, other):
        """See interfaces.IJSEventHandlers"""
        if not isinstance(other, JSHandlers):
            raise NotImplementedError
        handlers = self.copy()
        for field, event, handler in other._handlers:
            handlers.addHandler(field, event, handler)
        return handlers

    def __repr__(self):
        return '<%s %r>' % (self.__class__.__name__,
                            [handler for f, e, handler in self._handlers])


class JSHandler(object):
    zope.interface.implements(interfaces.IJSEventHandler)

    def __init__(self, func):
        self.func = func
        # XXX: should this ever be passed a string?
        # it is passed a string in unit tests, but I
        # think we may want to make that an invalid operation.
        if type(func) == str:
            self.__name__ = func
        else:
            self.__name__ = func.__name__

    def __call__(self, event, selector, request):
        return self.func(selector.widget.form, event, selector)

    def __repr__(self):
        return '<%s %r>' %(self.__class__.__name__, self.func)


def handler(field, event=jsevent.CLICK):
    """A decorator for defining a javascript event handler."""
    # As a convenience, we also accept form fields to the handler, but get the
    # real field immediately
    if IField.providedBy(field):
        field = field.field
    def createHandler(func):
        handler = JSHandler(func)
        frame = sys._getframe(1)
        f_locals = frame.f_locals
        handlers = f_locals.setdefault('jshandlers', JSHandlers())
        handlers.addHandler(field, event, handler)
        return func
    return createHandler


@zope.interface.implementer(zope.interface.Interface)
@zope.component.adapter(IAfterWidgetUpdateEvent)
def createSubscriptionsForWidget(event):
    widget = event.widget
    # Only react to widgets that have a field and know the form.
    if not (IFieldWidget.providedBy(widget) and IFormAware.providedBy(widget)):
        return
    # We only have work to do, if there are JS Handlers in the form.
    if not hasattr(widget.form, 'jshandlers'):
        return
    # Step 1: Get the handler.
    handlers = widget.form.jshandlers.getHandlers(widget.field)
    # Step 2: Create a selector.
    selector = WidgetSelector(widget)
    # Step 3: Make sure that the form has JS subscriptions, otherwise add
    #         it.
    if not interfaces.IHaveJSSubscriptions.providedBy(widget.form):
        widget.form.jsSubscriptions = jsevent.JSSubscriptions()
        zope.interface.alsoProvides(
            widget.form, interfaces.IHaveJSSubscriptions)
    # Step 4: Add the subscription to the form:
    for event, handler in handlers:
        widget.form.jsSubscriptions.subscribe(event, selector, handler)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Functions.

$Id$
"""
__docformat__ = "reStructuredText"
import sys
import zope.component
import zope.interface
from zope.interface import adapter

from zope.security.management import getInteraction
from zope.publisher.interfaces.browser import IBrowserRequest

from z3c.formjs import interfaces, jsfunction

CLIENT_EVENT_REQUEST_KEY = "z3c.formjs.jsclientevent.caughtEvents"


class ClientEventHandlers(object):
    """Client Handlers for server side events."""

    zope.interface.implements(interfaces.IClientEventHandlers)

    def __init__(self):
        self._registry = adapter.AdapterRegistry()
        self._handlers = ()

    def addHandler(self, required, handler):
        """See interfaces.IClientEventHandlers"""
        # Register the handler
        self._registry.subscribe(
            required, interfaces.IClientEventHandler, handler)
        self._handlers += ((required, handler),)

    def getHandlers(self, event):
        """See interfaces.IClientEventHandlers"""
        return self._registry.subscriptions(
            map(zope.interface.providedBy, (event.object, event)),
            interfaces.IClientEventHandler)

    def copy(self):
        """See interfaces.IClientEventHandlers"""
        handlers = ClientEventHandlers()
        for eventSpec, handler in self._handlers:
            handlers.addHandler(eventSpec, handler)
        return handlers

    def __add__(self, other):
        """See interfaces.IClientEventHandlers"""
        if not isinstance(other, ClientEventHandlers):
            raise NotImplementedError
        handlers = self.copy()
        for required, handler in other._handlers:
            handlers.addHandler(required, handler)
        return handlers

    def __repr__(self):
        return '<ClientEventHandlers %r>' % [
            handler for required, handler in self._handlers]


class ClientEventHandler(object):
    zope.interface.implements(interfaces.IClientEventHandler)

    def __init__(self, required, func):
        self.required = required
        self.func = func

    def __call__(self, form, event):
        return self.func(form, event)

    def __repr__(self):
        return '<%s for %r>' % (self.__class__.__name__, self.required)


def listener(required):
    """A decorator for defining a javascript function that is a listener."""
    def createListener(func):
        frame = sys._getframe(1)
        f_locals = frame.f_locals
        handlers = f_locals.setdefault(
            'jsClientListeners', ClientEventHandlers())
        jsListener = ClientEventHandler(required, func)
        return handlers.addHandler(required, jsListener)
    return createListener


@zope.component.adapter(zope.component.interfaces.IObjectEvent)
def serverToClientEventLoader(event):
    """Event handler that listens for server side events
    and stores the event in the request to be picked up by
    the form and rendered as a client side function call."""

    # Step 1: Get the interaction.
    try:
        interaction = getInteraction()
    except zope.security.interfaces.NoInteraction:
        # If there is no interaction, we simply return, because we cannot do
        # any work anyways. This also means that the event was not created
        # during a request, so we do not have a form anyways.
        return
    participations = interaction.participations

    # Step 2-1: Look for a request in the participation.
    request = None
    for part in participations:
        if IBrowserRequest.providedBy(part):
            request = part
            break
    # Step 2-2: If no request was found, we have nothing to do.
    if request is None:
        return
    # Step 3: Add the event to the list of events this handler has caught.
    events = request.annotations.setdefault(CLIENT_EVENT_REQUEST_KEY, [])
    if event not in events:
        events.append(event)


class ClientEventsForm(object):
    """Mixin class to support calling of client side events."""

    jsClientListeners = ClientEventHandlers()

    @property
    def eventCalls(self):
        events = self.request.annotations.get(CLIENT_EVENT_REQUEST_KEY, [])
        result = []
        for event in events:
            if self.jsClientListeners.getHandlers(event):
                result.append(event)
        return result

    @property
    def eventInjections(self):
        results = []
        for event in self.eventCalls:
            results += [h(self, event)
                        for h in self.jsClientListeners.getHandlers(event)]
        results = '\n'.join(results)
        return results



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Form Framework AJAX Framework.

$Id: $
"""
__docformat__ = "reStructuredText"
import sys
import zope.component
import zope.interface
from zope.publisher.interfaces import NotFound
from zope.publisher.browser import BrowserPage
from z3c.traverser import traverser
from z3c.form.util import SelectionManager
from z3c.traverser.interfaces import ITraverserPlugin
from z3c.formjs import interfaces


class AJAXHandlers(SelectionManager):
    """A selection manager for handling AJAX request handlers."""

    def addHandler(self, name, handler):
        self._data_keys.append(name)
        self._data_values.append(handler)
        self._data[name] = handler

    def __repr__(self):
        return "<%s %r>" % (self.__class__.__name__, self.keys())


class AJAXHandler(object):
    zope.interface.implements(interfaces.IAJAXHandler)

    def __init__(self, func):
        self.func = func

    def __call__(self, view):
        return self.func(view)

    def __repr__(self):
        return "<%s %r>" % (self.__class__.__name__, self.func.__name__)


class AJAXRequestHandler(object):
    """Mix-in class for forms to support AJAX calls."""
    zope.interface.implements(interfaces.IAJAXRequestHandler,
                              interfaces.IFormTraverser)

    ajaxRequestHandlers = AJAXHandlers()


def handler(func):
    """A decorator for defining an AJAX request handler."""
    handler = AJAXHandler(func)
    frame = sys._getframe(1)
    f_locals = frame.f_locals
    handlers = f_locals.setdefault('ajaxRequestHandlers', AJAXHandlers())
    handlers.addHandler(func.__name__, handler)
    return handler


class AJAXView(BrowserPage):
    """A wrapper class around AJAX handler to allow it to be publishable."""

    def __init__(self, handler, request, view):
        self.context = self.handler = handler
        self.request = request
        self.__parent__ = self.view = view

    def __call__(self):
        return self.handler(self.view)


class AJAXRequestTraverserPlugin(object):
    """Allow access to methods registered as an ajax request handler."""

    zope.interface.implements(ITraverserPlugin)

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def publishTraverse(self, request, name):
        handler = self.context.ajaxRequestHandlers.get(name)
        if handler is None:
            raise NotFound(self.context, name, request)

        return AJAXView(handler, self.request, self.context)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript Functions.

$Id: jsfunction.py 79157 2007-08-23 15:02:22Z srichter $
"""
__docformat__ = "reStructuredText"
import cStringIO
import compiler
import inspect
import sys
import zope.component
import zope.interface
from zope.viewlet import viewlet
from zope.publisher.interfaces.browser import IBrowserRequest

from z3c.formjs import interfaces

class JSFunction(object):
    zope.interface.implements(interfaces.IJSFunction)

    def __init__(self, namespace, function):
        self.namespace = namespace
        self.function = function

    @property
    def name(self):
        return self.function.func_name

    @property
    def arguments(self):
        args = inspect.getargspec(self.function)[0]
        if args[0] is 'self':
            del args[0]
        return args

    def render(self):
        return self.function(*[x for x in ['self'] + self.arguments])

    def call(self, *args):
        sargs = []
        for arg in args:
            argtype = type(arg)
            if argtype is unicode:
                sargs.append(repr(arg)[1:])
            elif argtype is bool:
                sargs.append(repr(arg).lower())
            elif argtype in (int, float, str):
                sargs.append(repr(arg))
            else:
                sargs.append(repr(str(arg)))
        caller = self.name
        if self.namespace:
            caller = '%s.%s' % (self.namespace, self.name)
        return '%s(%s);' % (caller, ','.join(sargs))

    def __repr__(self):
        return '<%s %s>' % (
            self.__class__.__name__, self.name)


class JSFunctions(object):
    zope.interface.implements(interfaces.IJSFunctions)

    def __init__(self):
        self._functions = {}

    def add(self, jsFunction, namespace=''):
        ns = self._functions.setdefault(namespace, [])
        ns.append(jsFunction)
        return jsFunction

    def render(self):
        result = ''
        # Render non-namespaced functions
        for func in self._functions.get('', []):
            args = func.arguments
            result += 'function %s(%s) {\n' %(
                func.name, ', '.join(args) )
            code = func.render()
            result += '  ' + code.replace('\n', '\n  ') + '\n'
            result += '}\n'
        # Render namespaced functions
        for ns, funcs in self._functions.items():
            if ns == '':
                continue
            result += 'var %s = {\n' %ns
            for func in funcs:
                args = func.arguments
                result += '  %s: function(%s) {\n' %(
                    func.name, ', '.join(args) )
                code = func.render()
                result += '    ' + code.replace('\n', '\n    ') + '\n'
                result += '  },\n'
            result = result[:-2] + '\n'
            result += '}\n'
        return result

    def __repr__(self):
        return '<%s %r>' % (self.__class__.__name__, self._functions)

def function(namespace=''):
    """A decorator for defining a javascript function."""
    def createFunction(func):
        frame = sys._getframe(1)
        f_locals = frame.f_locals
        funcs = f_locals.setdefault('jsFunctions', JSFunctions())
        jsFunction = JSFunction(namespace, func)
        return funcs.add(jsFunction, namespace)
    return createFunction


class JSFunctionsViewlet(viewlet.ViewletBase):
    """An viewlet for the JS viewlet manager rendering functions."""
    zope.component.adapts(
        zope.interface.Interface,
        IBrowserRequest,
        interfaces.IHaveJSFunctions,
        zope.interface.Interface)

    def render(self):
        content = self.__parent__.jsFunctions.render()
        return u'<script type="text/javascript">\n%s\n</script>' % content



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Javascript-based Value Validator

$Id: $
"""
__docformat__ = "reStructuredText"
import zope.interface
import zope.component
from zope.publisher.interfaces import NotFound

from z3c.form.interfaces import IWidget, IField

from z3c.formjs import ajax, interfaces

class BaseValidator(ajax.AJAXRequestHandler):
    zope.interface.implements(interfaces.IAJAXValidator)

    # See ``interfaces.IAJAXValidator``
    ValidationScript = None

    def _validate(self):
        shortName = self.request.get('widget-name')
        self.fields = self.fields.select(shortName)
        self.updateWidgets()
        return self.widgets.extract()


class MessageValidationScript(object):
    zope.interface.implements(interfaces.IMessageValidationScript)

    def __init__(self, form, widget):
        self.form = form
        self.widget = widget

    def render(self):
        renderer = zope.component.getMultiAdapter(
            (self, self.form.request), interfaces.IRenderer)
        return renderer.render()

class MessageValidator(BaseValidator):
    '''Validator that sends error messages for widget in questiodn.'''
    ValidationScript = MessageValidationScript

    @ajax.handler
    def validate(self):
        data, errors = self._validate()
        if errors:
            return errors[0].message
        return u'' # all OK



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Switching between display and edit mode.

$Id: jsswitch.py 79201 2007-08-24 01:52:26Z srichter $
"""
__docformat__ = "reStructuredText"
import zope.component
import zope.interface
import zope.schema.interfaces
from z3c.form import button
from z3c.form.interfaces import IWidgets, DISPLAY_MODE
from zope.publisher.interfaces import NotFound

from z3c.formjs import ajax, interfaces, jsevent, jsaction


class WidgetSwitcher(object):
    zope.interface.implements(interfaces.IWidgetSwitcher)

    def __init__(self, form, widget, mode):
        self.form = form
        self.widget = widget
        self.mode = mode

    def render(self):
        renderer = zope.component.getMultiAdapter(
            (self, self.form.request), interfaces.IRenderer)
        return renderer.render()

class LabelWidgetSwitcher(object):
    zope.interface.implements(interfaces.ILabelWidgetSwitcher)

    def __init__(self, request, mode):
        self.request = request
        self.mode = mode

    def render(self):
        renderer = zope.component.getMultiAdapter(
            (self, self.request), interfaces.IRenderer)
        return renderer.render()


class WidgetSaver(object):
    zope.interface.implements(interfaces.IWidgetSaver)

    switcherClass = WidgetSwitcher

    def __init__(self, form, widget):
        self.form = form
        self.widget = widget
        self.switcher = self.switcherClass(form, widget, DISPLAY_MODE)

    def render(self):
        renderer = zope.component.getMultiAdapter(
            (self, self.form.request), interfaces.IRenderer)
        return renderer.render()


class WidgetModeSwitcher(ajax.AJAXRequestHandler):
    """A mix-in to forms to allow switching between widget modes."""
    zope.interface.implements(interfaces.IWidgetModeSwitcher,
                              interfaces.IHaveJSSubscriptions)
    buttons = button.Buttons()
    mode = DISPLAY_MODE

    @jsaction.handler(zope.schema.interfaces.IField, jsevent.CLICK)
    def switchToInputWidget(self, event, selector):
        return WidgetSwitcher(self, selector.widget, 'input').render()

    @jsaction.handler(zope.schema.interfaces.IField, jsevent.BLUR)
    def saveWidgetValue(self, event, selector):
        return WidgetSaver(self, selector.widget).render()

    @jsevent.subscribe(jsevent.CSSSelector('label'), jsevent.CLICK)
    def labelClickSwitchesToInputWidget(event, selector, request):
        return LabelWidgetSwitcher(request, 'input').render()

    def _getWidget(self, mode):
        """Get the widget in the given mode."""
        # There are two possibilities to extract the wdget, either by widget
        # short name or by widget id. The latter is significantly more
        # inefficient, but needed.
        if 'widget-id' in self.request.form:
            # Step 1: Determine the full name of the widget
            name = self.request.form['widget-id'].replace('-', '.')
            # Step 2: Limit the form fields only to this one widget.
            # Note: This algorithm might not be fully reliable, but should
            #       work for now.
            for shortName in self.fields:
                if name.endswith(shortName):
                    break
        else:
            # Step 1: Determine the name of the widget.
            shortName = self.request.form['widget-name']
            # Step 2: Limit the form fields only to this one widget.
            self.fields = self.fields.select(shortName)
            # Step 3: Instantiate the widget manager, set the correct mode and
            #         update it.
            self.widgets = zope.component.getMultiAdapter(
                (self, self.request, self.getContent()), IWidgets)
        # Step 3: Instantiate the widget manager, set the correct mode and
        #         update it.
        self.widgets = zope.component.getMultiAdapter(
            (self, self.request, self.getContent()), IWidgets)
        self.widgets.mode = mode
        self.widgets.update()
        # Step 4: Return the widget.
        return self.widgets[shortName]

    @ajax.handler
    def getDisplayWidget(self):
        '''See interfaces.IWidgetModeSwitcher'''
        widget = self._getWidget('display')
        handlers = dict(widget.form.jshandlers.getHandlers(widget.field))
        code = handlers[jsevent.CLICK](
            jsevent.CLICK, jsaction.WidgetSelector(widget), self.request)
        widget.onclick = unicode(code.replace('\n', ' '))
        return widget.render()

    @ajax.handler
    def getInputWidget(self):
        '''See interfaces.IWidgetModeSwitcher'''
        widget = self._getWidget('input')
        handlers = dict(widget.form.jshandlers.getHandlers(widget.field))
        code = handlers[jsevent.BLUR](
            jsevent.BLUR, jsaction.WidgetSelector(widget), self.request)
        widget.onblur = unicode(code.replace('\n', ' '))
        return widget.render()

    @ajax.handler
    def saveWidgetValue(self):
        '''See interfaces.IWidgetModeSwitcher'''
        widget = self._getWidget('input')
        data, errors = self.extractData()
        if errors:
            return errors[0].message
        self.applyChanges(data)
        return ''




##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""JQuery-backend implementation

$Id: jqueryrenderer.py 79201 2007-08-24 01:52:26Z srichter $
"""
__docformat__ = "reStructuredText"
import zope.component
import zope.interface
from zope.traversing.browser import absoluteURL

from jquery.layer import IJQueryJavaScriptBrowserLayer

from z3c.formjs import interfaces


class JQueryIdSelectorRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(
        interfaces.IIdSelector, IJQueryJavaScriptBrowserLayer)

    def __init__(self, selector, request):
        self.selector = selector

    def update(self):
        pass

    def render(self):
        return u'#' + self.selector.id


class JQueryCSSSelectorRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(
        interfaces.ICSSSelector, IJQueryJavaScriptBrowserLayer)

    def __init__(self, selector, request):
        self.selector = selector

    def update(self):
        pass

    def render(self):
        return self.selector.expr


class JQuerySubscriptionRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(
        interfaces.IJSSubscription, IJQueryJavaScriptBrowserLayer)

    def __init__(self, subscription, request):
        self.subscription = subscription
        self.request = request

    def update(self):
        self.selectorRenderer = zope.component.getMultiAdapter(
            (self.subscription.selector, self.request), interfaces.IRenderer)
        self.selectorRenderer.update()

    def render(self):
        return u'$("%s").bind("%s", function(event){%s});' %(
            self.selectorRenderer.render(),
            self.subscription.event.name,
            self.subscription.handler(
                self.subscription.event,
                self.subscription.selector,
                self.request) )


class JQuerySubscriptionsRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(
        interfaces.IJSSubscriptions, IJQueryJavaScriptBrowserLayer)

    def __init__(self, manager, request):
        self.manager = manager
        self.request = request

    def update(self):
        self.renderers = []
        for subscription in self.manager:
            renderer = zope.component.getMultiAdapter(
                (subscription, self.request), interfaces.IRenderer)
            renderer.update()
            self.renderers.append(renderer)

    def render(self):
        return '$(document).ready(function(){\n  %s\n})' %(
            '\n  '.join([r.render() for r in self.renderers]) )


class JQueryMessageValidationScriptRenderer(object):
    zope.component.adapts(
        interfaces.IMessageValidationScript, IJQueryJavaScriptBrowserLayer)
    zope.interface.implements(interfaces.IRenderer)

    function = 'applyErrorMessage'

    def __init__(self, script, request):
        self.context = script
        self.request = request

    def _ajaxURL(self):
        widget = self.context.widget
        form = self.context.form
        # build js expression for extracting widget value
        valueString = '$("#%s").val()' % widget.id
        # build a js expression that joins valueString expression
        queryString = '"?widget-name=%s&%s=" + %s' % (
            widget.__name__, widget.name, valueString)
        # build a js expression that joins form url, validate path, and query
        # string
        ajaxURL = '"'+form.request.getURL() + '/@@ajax/validate" + ' \
                  + queryString

        return ajaxURL

    def update(self):
        pass

    def render(self):
        ajaxURL = self._ajaxURL()
        # build a js expression that shows the user the error message
        widget = self.context.widget
        messageSetter = '%s("%s", msg)' % (self.function, widget.id)
        ajax = '$.get(%s,\nfunction(msg){%s}\n)' % (ajaxURL, messageSetter)
        return ajax


class JQueryWidgetSwitcherRenderer(object):
    zope.component.adapts(
        interfaces.IWidgetSwitcher, IJQueryJavaScriptBrowserLayer)
    zope.interface.implements(interfaces.IRenderer)

    function = 'switchWidget'

    def __init__(self, switcher, request):
        self.context = switcher
        self.request = request

    def _ajaxURL(self):
        widget = self.context.widget
        form = self.context.form
        # build a js expression that joins valueString expression
        queryString = '?widget-name=%s' % widget.__name__
        mode = self.context.mode.title()
        # build a js expression that joins form url, validate path, and query
        # string
        ajaxURL = '"'+ absoluteURL(form, form.request) + '/@@ajax/get' \
                  + mode + 'Widget' + queryString + '"'

        return ajaxURL

    def update(self):
        pass

    def render(self):
        ajaxURL = self._ajaxURL()
        widget = self.context.widget
        switcherCall = '%s("%s", "%s", html)' % (
            self.function, widget.id, self.context.mode)
        return '$.get(%s,\nfunction(html){%s}\n)' % (ajaxURL, switcherCall)


class JQueryLabelWidgetSwitcherRenderer(object):
    zope.component.adapts(
        interfaces.ILabelWidgetSwitcher, IJQueryJavaScriptBrowserLayer)
    zope.interface.implements(interfaces.IRenderer)

    function = 'switchWidget'
    widgetIdExpr = 'event.target.parentNode.attributes["for"].value'

    def __init__(self, switcher, request):
        self.context = switcher
        self.request = request

    def _ajaxURL(self):
        # build a js expression that joins valueString expression
        queryString = '?widget-id=' + self.widgetIdExpr
        mode = self.context.mode.title()
        # build a js expression
        return '"%s/@@ajax/get%sWidget?widget-id=" + %s' % (
            self.request.getURL(), mode, self.widgetIdExpr)

    def update(self):
        pass

    def render(self):
        ajaxURL = self._ajaxURL()
        switcherCall = '%s(%s, "%s", html)' % (
            self.function, self.widgetIdExpr, self.context.mode)
        return '$.get(%s,\nfunction(html){%s}\n)' % (ajaxURL, switcherCall)


class JQueryWidgetSaverRenderer(object):
    zope.component.adapts(
        interfaces.IWidgetSaver, IJQueryJavaScriptBrowserLayer)
    zope.interface.implements(interfaces.IRenderer)

    function = 'saveWidget'

    def __init__(self, switcher, request):
        self.context = switcher
        self.request = request

    def _ajaxURL(self):
        widget = self.context.widget
        form = self.context.form
        # build js expression for extracting widget value
        valueString = '$("#%s").val()' % widget.id
        # build a js expression that joins valueString expression
        queryString = '?widget-name=%s&%s=" + %s' % (
            widget.__name__, widget.name, valueString)
        # build a js expression that joins form url, validate path, and query
        # string
        ajaxURL = '"'+ absoluteURL(form, form.request) + \
                  '/@@ajax/saveWidgetValue' + queryString

        return ajaxURL

    def update(self):
        pass

    def render(self):
        ajaxURL = self._ajaxURL()
        widget = self.context.widget
        saveCall = '%s("%s", msg)' % (self.function, widget.id)
        switchCall = 'if (error == false) {%s}' %self.context.switcher.render()
        return '$.get(%s,\nfunction(msg){error=%s; %s}\n)' % (
            ajaxURL, saveCall, switchCall)



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Common z3c.formks test setups

$Id: $
"""
__docformat__ = 'restructuredtext'
import os.path
import zope.component
import zope.interface
from zope.app.pagetemplate import viewpagetemplatefile
from zope.app.testing import setup
from zope.publisher.interfaces.browser import IBrowserRequest

import z3c.formjs.tests
from z3c.formjs import interfaces

class IdSelectorRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(interfaces.IIdSelector, IBrowserRequest)

    def __init__(self, selector, request):
        self.selector = selector

    def update(self):
        pass

    def render(self):
        return u'#' + self.selector.id

class CSSSelectorRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(interfaces.ICSSSelector, IBrowserRequest)

    def __init__(self, selector, request):
        self.selector = selector

    def update(self):
        pass

    def render(self):
        return unicode(self.selector.expr)

class SubscriptionRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(interfaces.IJSSubscription, IBrowserRequest)

    def __init__(self, subscription, request):
        self.subscription = subscription
        self.request = request

    def update(self):
        self.selectorRenderer = zope.component.getMultiAdapter(
            (self.subscription.selector, self.request), interfaces.IRenderer)
        self.selectorRenderer.update()

    def render(self):
        return u'$("%s").bind("%s", function(){%s});' %(
            self.selectorRenderer.render(),
            self.subscription.event.name,
            self.subscription.handler(
                self.subscription.event,
                self.subscription.selector,
                self.request) )

class ManagerRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(interfaces.IJSSubscriptions, IBrowserRequest)

    def __init__(self, manager, request):
        self.manager = manager
        self.request = request

    def update(self):
        self.renderers = []
        for subscription in self.manager:
            renderer = zope.component.getMultiAdapter(
                (subscription, self.request), interfaces.IRenderer)
            renderer.update()
            self.renderers.append(renderer)

    def render(self):
        return '$(document).ready(function(){\n  %s\n})' %(
            '\n  '.join([r.render() for r in self.renderers]) )


class MessageValidationScriptRenderer(object):
    zope.interface.implements(interfaces.IRenderer)
    zope.component.adapts(
        interfaces.IMessageValidationScript, IBrowserRequest)

    def __init__(self, script, request):
        self.script = script
        self.request = request

    def render(self):
        return "$.get('/validate', function(data){ alert(data) })"


class WidgetSwitcherRenderer(object):
    zope.component.adapts(interfaces.IWidgetSwitcher, IBrowserRequest)
    zope.interface.implements(interfaces.IRenderer)

    def __init__(self, switcher, request):
        self.context = switcher
        self.request = request

    def update(self):
        pass

    def render(self):
        widget = self.context.widget
        switcherCall = 'switchWidget("%s", html)' % (widget.id)
        return "$.get('/switchWidget', function(html){%s}\n)" % switcherCall


class LabelWidgetSwitcherRenderer(object):
    zope.component.adapts(
        interfaces.ILabelWidgetSwitcher, IBrowserRequest)
    zope.interface.implements(interfaces.IRenderer)

    widgetIdExpr = 'event.target.parentNode.attributes["for"].value'

    def __init__(self, switcher, request):
        self.context = switcher
        self.request = request

    def update(self):
        pass

    def render(self):
        switcherCall = 'switchWidget(%s, html)' % (self.widgetIdExpr)
        return "$.get('/switchWidget', function(html){%s}\n)" % switcherCall


class WidgetSaverRenderer(object):
    zope.component.adapts(
        interfaces.IWidgetSaver, IBrowserRequest)
    zope.interface.implements(interfaces.IRenderer)

    def __init__(self, switcher, request):
        self.context = switcher
        self.request = request

    def update(self):
        pass

    def render(self):
        widget = self.context.widget
        saveCall = 'saveWidget("%s", msg)' % widget.id
        return "$.get('saveValue', function(msg){%s}\n)" % saveCall


def setupRenderers():
    zope.component.provideAdapter(IdSelectorRenderer)
    zope.component.provideAdapter(CSSSelectorRenderer)
    zope.component.provideAdapter(SubscriptionRenderer)
    zope.component.provideAdapter(ManagerRenderer)
    zope.component.provideAdapter(MessageValidationScriptRenderer)
    zope.component.provideAdapter(WidgetSwitcherRenderer)
    zope.component.provideAdapter(LabelWidgetSwitcherRenderer)
    zope.component.provideAdapter(WidgetSaverRenderer)


def addTemplate(form, filename):
    path = os.path.join(os.path.dirname(z3c.formjs.tests.__file__), filename)
    form.template = viewpagetemplatefile.BoundPageTemplate(
        viewpagetemplatefile.ViewPageTemplateFile(path), form)


def getPath(filename):
    return os.path.join(os.path.dirname(browser.__file__), filename)

def setUp(test):
    setup.placelessSetUp(test)

def tearDown(test):
    setup.placelessTearDown(test)
